from pathlib import Path

import click
from ruamel.yaml import YAML, scanner

from backups_rotate.area import Area

DEFAULT_CONFIG_FILE = "/etc/backups-rotate.yaml"

@click.command()
@click.option(
    "--config",
    "-c",
    default=DEFAULT_CONFIG_FILE,
    help="Configuration file",
)
@click.option(
    "--list",
    "-l",
    is_flag=True,
    help="List areas",
)
@click.option(
    "--list-deleted",
    "-d",
    is_flag=True,
    help="list only files to be deleted",
)
@click.option(
    "--verbose",
    "-v",
    is_flag=True,
    help="Verbose mode",
)
@click.option(
    "--dry-run",
    "-n",
    is_flag=True,
    help="Dry run",
)
@click.argument('area_names', nargs=-1)
def cli(config, list, list_deleted, verbose, dry_run, area_names):
    """
    backups_rotate: a tool to manage backups
    """

    # read the configuration
    yaml = YAML(typ='safe')
    candidates = [ Path(config), Path(".") / Path(config).name ]
    for candidate in candidates:
        if verbose:
            click.echo(f"Trying to reading configuration from {candidate}")
        if candidate.exists():
            try:
                with open(candidate) as f:
                    config = yaml.load(f)
                    areas_by_name = {area["name"]: area for area in config}
                if verbose:
                    click.echo(f"Configuration read from {candidate}")
                break
            except scanner.ScannerError as e:
                click.echo(f"Error reading configuration from {candidate}: {e}")
    else:
        click.echo(f"Config file not found - exiting - have tried:")
        for candidate in candidates:
            click.echo(f"  {candidate.absolute()}")
        return 1

    if not area_names:
        area_dicts = config
    else:
        try:
            area_dicts = [areas_by_name[name] for name in area_names]
        except KeyError as e:
            click.echo(f"Area {e} not found in configuration")
            return 1
    try:
        areas = [Area(area_dict) for area_dict in area_dicts]
    except ValueError as e:
        click.echo(f"Error creating area: {e}")
        return 1

    for area in areas:
        area.read()
        if verbose:
            click.echo(f"area={area}")
        if list:
            kept = not list_deleted
            area.list(verbose=verbose, kept=kept)
        else:
            area.delete(dry_run=dry_run, verbose=verbose)


if __name__ == "__main__":
    cli()



# pylint: disable=missing-docstring

POLICY1 = {
    'hour': 'infinite',
}
# wrt sample1 we expect 44 entries
POLICY1_SAMPLE1 = 44
# all entries are kept with sample 2
POLICY1_SAMPLE2 = 7680

POLICY2 = {
    'year': 1,
}
POLICY2_SAMPLE1 = 1
POLICY2_SAMPLE2 = 1

POLICY3 = {
    'year': 'infinite',
}
POLICY3_SAMPLE1 = 4
POLICY3_SAMPLE2 = 1

POLICY4 = {
    'quarter': 3,
}
POLICY4_SAMPLE1 = 3
POLICY4_SAMPLE2 = 3


POLICY10 = {
    'year': 'infinite',
    'quarter': 3,
    'month': 2,
    'week': 4,
    'day': 5,
    'hour': 6,
}

# wrt sample1 we expect a maximum of
# - 4 years, 3 quarters, 2 months, 4 weeks, 5 days, 6 hours
# i.e. 24
# except that the latest event (#3) appears 6 times - so -5
# the last event of year 2023 (#23) appears for year and quarter so -1
# event #8 (2024-03-31) appears for quarter, month, week and day, so -3
# event #29 (2024-02-03) appears for week and day, so -1
# so bottom line is 24 - 5 - 1 - 3 - 1 = 14
POLICY10_SAMPLE1 = 14

# wrt sample2 we have one entry every hour between jan 1 and nov 17 2024
# so we expect 3+2+4+5+6+1 = 21, minus:
# very last event is present 6 times, so -5
# all other dimensions are orthogonal, so that's it
# 21 - 5 = 16
POLICY10_SAMPLE2 = 16


POLICY11 = POLICY10.copy()
POLICY11['month'] = 'infinite'

# one additional result for january to september
# but june and march were already picked for quarter
POLICY11_SAMPLE2 = POLICY10_SAMPLE2 + 9 - 2
